/*
	CRC32 / FPGA calcrator via serial port

	Win32 version
*/

#include <windows.h>
#include "stdio.h"

static HANDLE com_hndl;


/////////////////////////////////////////////////////////////////////////////
// COM|[gI[v
/////////////////////////////////////////////////////////////////////////////
/* RS232C interface */
FILE *com_open(const char *device,int baudrate)
{
	BOOL success;
	DCB dcb;
	char filename[256];
	DWORD fdwCreate;
	HANDLE hndl;
	DWORD dwDesiredAccess = 0;
	DWORD dwCreationDisposition = 0;

	SECURITY_ATTRIBUTES sa;
	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.lpSecurityDescriptor = 0;
	sa.bInheritHandle = TRUE;

	// convert device name to support COM10+
	sprintf(filename,"\\\\.\\%s",device);
	fdwCreate = OPEN_EXISTING;
	dwDesiredAccess = GENERIC_READ | GENERIC_WRITE;
	hndl = CreateFile(filename,dwDesiredAccess,0,&sa,fdwCreate,FILE_ATTRIBUTE_NORMAL,0);

	if(hndl != INVALID_HANDLE_VALUE)
	{
		success = GetCommState(hndl,&dcb);
		if(success)
		{
			dcb.BaudRate     = baudrate;
			dcb.ByteSize     = 8;
			dcb.fBinary      = TRUE;
			dcb.fParity      = FALSE;
			dcb.fOutxCtsFlow = TRUE;
			dcb.fOutxDsrFlow = FALSE;
			dcb.fDtrControl  = DTR_CONTROL_ENABLE;
			dcb.fDsrSensitivity = FALSE;
			dcb.fTXContinueOnXoff = FALSE;
			dcb.fNull        = FALSE;
			dcb.fRtsControl  = RTS_CONTROL_HANDSHAKE;
			dcb.Parity       = NOPARITY;
			dcb.StopBits     = ONESTOPBIT;
			success = SetCommState(hndl,&dcb);
			if(success)
			{
				SetCommMask(hndl,EV_TXEMPTY);
				return (FILE *)(hndl);
			}
		}
		CloseHandle(hndl);
		hndl = INVALID_HANDLE_VALUE;
	}
	return 0;
}

int com_write(FILE *h,void *data,int size)
{
	HANDLE hndl = (HANDLE)h;
	DWORD write_size;

	if(hndl != INVALID_HANDLE_VALUE)
	{
		if( WriteFile(hndl,data,size,&write_size,NULL) == TRUE )
			return write_size;
		return 0;
	}
	return -1;
}

int com_read(FILE *h,void *data,int size)
{
	HANDLE hndl = (HANDLE)h;
	DWORD read_size;

	if(hndl != INVALID_HANDLE_VALUE)
	{
		if( ReadFile(hndl,data,size,&read_size,NULL) == TRUE )
			return read_size;
		return 0;
	}
	return -1;
}

void com_close(FILE *h)
{
	HANDLE hndl = (HANDLE)h;
	CloseHandle(hndl);
}

void com_send_break(FILE *h)
{
	HANDLE hndl = (HANDLE)h;

	Sleep(100);
	SetCommBreak(hndl);
	Sleep(100);
	ClearCommBreak(hndl);
	Sleep(100);

}

void com_clear_receive_buffer(FILE *h)
{
	HANDLE hndl = (HANDLE)h;
	PurgeComm(hndl,PURGE_RXCLEAR);
}

/////////////////////////////////////////////////////////////////////////////
// t@CCRCvZ
/////////////////////////////////////////////////////////////////////////////
void fpga_crc_calc(
	const char *filepath,
	int calc_size,
	unsigned int crc_initial,
	unsigned int crc_polynomial,
	unsigned int *crc
)
{
	unsigned char com_buffer[8];
	unsigned char crc_din;
	int num_calc;
	FILE *fp,*fpcom;

	// t@C̃I[v
	fp = fopen(filepath,"rb");
	if(fp==0)
	{
		printf("Can't open data file\n");
		return;
	}
	// VA|[g̃I[vƐݒ
	fpcom = com_open("COM1",115200);
	if(fpcom==0)
	{
		printf("Can't open COM port\n");
		return;
	}

	// u[NMCRC32HZbg
	com_send_break(fpcom);

	// Əl𑗐M
	com_buffer[0] = crc_polynomial>>24;
	com_buffer[1] = crc_polynomial>>16;
	com_buffer[2] = crc_polynomial>>8;
	com_buffer[3] = crc_polynomial>>0;
	com_buffer[4] = crc_initial>>24;
	com_buffer[5] = crc_initial>>16;
	com_buffer[6] = crc_initial>>8;
	com_buffer[7] = crc_initial>>0;
	com_write(fpcom,com_buffer,8);

	// t@Cŏ̂PoCg[h
	for(num_calc=0;num_calc<calc_size;num_calc++)
	{
		if( (crc_din=fgetc(fp)) <0) break;
		// PoCgCRCvZ
		com_write(fpcom,&crc_din,1);
	}

	// Mf[^NAĂ
	com_clear_receive_buffer(fpcom);
	// u[NMăf[^M
	com_send_break(fpcom);
	// CRC32ʂM
	com_read(fpcom,com_buffer,4);

	// CRC32lM
	// com_send_break(fpcom);
	*crc = 
		(com_buffer[0]<<24) |
		(com_buffer[1]<<16) |
		(com_buffer[2]<<8) |
		(com_buffer[3]<<0);

	com_close(fpcom);
	fclose(fp);
}

