//#include <iostream>
#include <stdio.h>
#include <verilated.h>          // Defines common routines
#if VM_TRACE
#include <verilated_vcd_c.h>	// Trace file format header
#endif
#include "Vcrc32_sync8__Syms.h"       // From Verilating "sim_crc32s_file.v"

//#define VCD_FILEPARH "vlt_dump.vcd"
#define VCD_FILEPARH "NUL"

#define SIMULATE_FILEPARH "c:\\rand10mb.bin"
//#define SIMURATE_BYTES (100*1024*1024)
//#define SIMURATE_BYTES (1*1024*1024)
#define SIMURATE_BYTES (50*1024)

unsigned int main_time = 0;     // Current simulation time

double sc_time_stamp () {       // Called by $time in Verilog
    return main_time;
}

//
Vcrc32_sync8 *top;

#if VM_TRACE
VerilatedVcdC* tfp;
#endif

// VA|[gڑꂽFPGAC{[hɂCRC32vZ֐
extern void fpga_crc_calc(
	const char *filepath,
	int calc_size,
	unsigned int crc_initial,
	unsigned int crc_polynomial,
	unsigned int *crc
);

// C++R[hɂCRC32vZ֐
extern void cpp_crc_calc(
	const char *filepath,
	int calc_size,
	unsigned int crc_initial,
	unsigned int crc_polynomial,
	unsigned int *crc
);

/////////////////////////////////////////////////////////////////////////////
// ̑I
/////////////////////////////////////////////////////////////////////////////
#define POLYNOMIAL_CRC32_IEEE802_3 0x04C11DB7
#define POLYNOMIAL_CRC16_CCITT     0x10210000

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
void pulse_clock(void)
{
	top->eval();
#if VM_TRACE
	if (tfp) tfp->dump (main_time);	// Create waveform trace for this timestamp
#endif
	main_time++;
	top->I_CLK   = 1;
	top->eval();
#if VM_TRACE
	if (tfp) tfp->dump (main_time);	// Create waveform trace for this timestamp
#endif
	main_time++;
	top->I_CLK   = 0;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
void crc32_byte_out(int byte_data)
{
	// f[^ZbgAbv
	top->I_SIN8  = byte_data;
	top->I_START   = 1;
	pulse_clock();
	top->I_START   = 0; // 󂯕t猳ɖ߂
	// vZI܂ł܂iWNbNj
	while(top->O_READY!=1)
		pulse_clock();
	//
	top->I_CRC32 = top->O_CRC32;

}

/////////////////////////////////////////////////////////////////////////////
// t@CCRCvZ
/////////////////////////////////////////////////////////////////////////////
void crc_file_calc(
	FILE *fp,
	int bit_size,
	unsigned int crc_initial,
	unsigned int crc_polynomial,
	unsigned int *crc
)
{
	int crc_din;
	int num_calc;

	// Əlݒ
	top->I_FEED  = crc_polynomial;
	top->I_CRC32 = crc_initial;
	top->I_LOAD  = 1;
	pulse_clock();

	// t@Cŏ̂PoCg[h
	for(num_calc=0;num_calc<SIMURATE_BYTES;num_calc++)
	{
		if( (crc_din=fgetc(fp)) <0) break;
		// PoCgCRCvZ
		crc32_byte_out(crc_din);
//		printf("byte %02X,CRC32=%08X\n",crc_din,(top->O_CRC32)>>(32-bit_size) );
	}
	*crc = top->O_CRC32;
	printf("Finish Verilated VerilogHDL CRC calcrate\n");
}

/////////////////////////////////////////////////////////////////////////////
// C
/////////////////////////////////////////////////////////////////////////////
// ̑I
#define POLYNOMIAL_CRC32_IEEE802_3 0x04C11DB7
#define POLYNOMIAL_CRC16_CCITT     0x10210000

int main(int argc, char** argv) {
	FILE *fp;
	unsigned int crc32,crc16;

	top = new Vcrc32_sync8();       // Create instance

    Verilated::commandArgs(argc, argv);   // Remember args
    Verilated::debug(0);
#if VM_TRACE			// If verilator was invoked with --trace
    Verilated::traceEverOn(true);	// Verilator must compute traced signals
    VL_PRINTF("Enabling waves...\n");
    tfp = new VerilatedVcdC;
    top->trace (tfp, 1);	// Trace levels of hierarchy
    tfp->open (VCD_FILEPARH);	// Open the dump file
#endif

	top->I_RESET = 1;
	top->I_CLK   = 0;
	top->I_FEED  = 0;
	top->I_CRC32 = 0;
	top->I_SIN8  = 0;
	top->I_START = 0;
	top->I_LOAD  = 0;

	pulse_clock();
	pulse_clock();
	top->I_RESET = 0;

	// VerilogHDL - Verilated Simulation
	fp = fopen(SIMULATE_FILEPARH,"rb");
	if(fp==0)
	{
		printf("Can't open in file\n");
		return -1;
	}
	crc_file_calc(fp,32,0xffffffff,POLYNOMIAL_CRC32_IEEE802_3,&crc32);
	fclose(fp);

	fp = fopen(SIMULATE_FILEPARH,"rb");
	if(fp==0)
	{
		printf("Can't open in file\n");
		return -1;
	}
	crc_file_calc(fp,16,0xffff0000,POLYNOMIAL_CRC16_CCITT,&crc16);
	fclose(fp);
	printf("VerilogHDL - CRC32 calcrate\n");
	printf("CRC32 = %08X\n",crc32);
	printf("CRC16 = %04X\n",crc16>>16);

	// C++ crc calcration
	cpp_crc_calc(SIMULATE_FILEPARH,SIMURATE_BYTES,0xffffffff,POLYNOMIAL_CRC32_IEEE802_3,&crc32);
	printf("Finish C++ CRC32 calcrate\n");
	cpp_crc_calc(SIMULATE_FILEPARH,SIMURATE_BYTES,0xffff0000,POLYNOMIAL_CRC16_CCITT    ,&crc16);
	printf("Finish C++ CRC16 calcrate\n");
	printf("CRC32 = %08X\n",crc32);
	printf("CRC16 = %04X\n",crc16>>16);

	// FPGA crc calcration
	fpga_crc_calc(SIMULATE_FILEPARH,SIMURATE_BYTES,0xffffffff,POLYNOMIAL_CRC32_IEEE802_3,&crc32);
	printf("Finish FPGA CRC32 calcrate\n");
	fpga_crc_calc(SIMULATE_FILEPARH,SIMURATE_BYTES,0xffff0000,POLYNOMIAL_CRC16_CCITT    ,&crc16);
	printf("Finish FPGA CRC16 calcrate\n");
	printf("CRC32 = %08X\n",crc32);
	printf("CRC16 = %04X\n",crc16>>16);

#if VM_TRACE
    if (tfp) tfp->close();
#endif

}
